@@ -12,62 +12,65 @@ struct kino_VArray {
     KINO_VARRAY_VTABLE *_;
     KINO_OBJ_MEMBER_VARS;
     kino_Obj   **elems;
-    kino_u32_t   size;
-    kino_u32_t   cap;
+    chy_u32_t    size;
+    chy_u32_t    cap;
 };
 
 /* Constructor.
  */
-KINO_FUNCTION(
 kino_VArray*
-kino_VA_new(kino_u32_t capacity));
+kino_VA_new(chy_u32_t capacity);
 
 /* Push an item onto the end of a VArray.
  */
-KINO_METHOD("Kino_VA_Push",
 void
-kino_VA_push(kino_VArray *self, kino_Obj *element));
+kino_VA_push(kino_VArray *self, kino_Obj *element);
+KINO_METHOD("Kino_VA_Push");
 
 /* Pop an item off of the end of a VArray.
  */
-KINO_METHOD("Kino_VA_Pop",
 kino_Obj*
-kino_VA_pop(kino_VArray *self));
+kino_VA_pop(kino_VArray *self);
+KINO_METHOD("Kino_VA_Pop");
 
 /* Unshift an item onto the front of a VArray.
  */
-KINO_METHOD("Kino_VA_Unshift",
 void
-kino_VA_unshift(kino_VArray *self, kino_Obj *element));
+kino_VA_unshift(kino_VArray *self, kino_Obj *element);
+KINO_METHOD("Kino_VA_Unshift");
 
 /* Shift an item off of the front of a VArray.
  */
-KINO_METHOD("Kino_VA_Shift",
 kino_Obj*
-kino_VA_shift(kino_VArray *self));
+kino_VA_shift(kino_VArray *self);
+KINO_METHOD("Kino_VA_Shift");
 
 /* Ensure that the VArray has room for at least [capacity] elements.
  */
-KINO_METHOD("Kino_VA_Grow",
 void
-kino_VA_grow(kino_VArray *self, kino_u32_t capacity));
+kino_VA_grow(kino_VArray *self, chy_u32_t capacity);
+KINO_METHOD("Kino_VA_Grow");
 
 /* Fetch the element at index [num].
  */
-KINO_METHOD("Kino_VA_Fetch",
 kino_Obj*
-kino_VA_fetch(kino_VArray *self, kino_u32_t num));
+kino_VA_fetch(kino_VArray *self, chy_u32_t num);
+KINO_METHOD("Kino_VA_Fetch");
 
 /* Store an element at index [num].  Any existing element will have its
  * refcount decremented.
  */
-KINO_METHOD("Kino_VA_Store",
 void
-kino_VA_store(kino_VArray *self, kino_u32_t num, kino_Obj *elem));
+kino_VA_store(kino_VArray *self, chy_u32_t num, kino_Obj *elem);
+KINO_METHOD("Kino_VA_Store");
+
+kino_VArray*
+kino_VA_clone(kino_VArray *self);
+KINO_METHOD("Kino_VA_Clone");
 
-KINO_METHOD("Kino_VA_Destroy",
 void
-kino_VA_destroy(kino_VArray *self));
+kino_VA_destroy(kino_VArray *self);
+KINO_METHOD("Kino_VA_Destroy");
 
 KINO_END_CLASS
 
