@@ -7,19 +7,18 @@ use base qw( KinoSearch::Util::Class );
 
 use Time::HiRes qw( time );
 
-BEGIN {
-    __PACKAGE__->init_instance_vars(
-        # params/members
-        schema => undef,
+our %instance_vars = (
+    # params/members
+    schema => undef,
 
-        # members
-        infos       => {},
-        seg_counter => 0,
-        version     => ( int( time * 1000 ) ),
-        generation  => 0,
-    );
-    __PACKAGE__->ready_get_set(qw( seg_counter generation ));
-}
+    # members
+    infos       => {},
+    seg_counter => 0,
+    version     => ( int( time * 1000 ) ),
+    generation  => 0,
+);
+
+BEGIN { __PACKAGE__->ready_get_set(qw( seg_counter generation )) }
 
 use KinoSearch::Index::SegInfo;
 use KinoSearch::Schema::FieldSpec;
@@ -31,8 +30,8 @@ use KinoSearch::Util::YAML qw( encode_yaml parse_yaml );
 
 sub init_instance {
     my $self = shift;
-    confess("Missing required parameter schema") unless defined
-        $self->{schema};
+    confess("Missing required parameter schema")
+        unless defined $self->{schema};
 }
 
 # Add a SegInfo to the collection.
@@ -64,11 +63,24 @@ sub infos {
     values %{ $_[0]->{infos} };
 }
 
+my %read_infos_args = (
+    folder   => undef,
+    filename => undef,
+);
+
 # Decode "segments" file.
 sub read_infos {
-    my ( $self, $folder ) = @_;
-    my $schema   = $self->{schema};
-    my $filename = $folder->latest_gen( 'segments', '.yaml' );
+    my $self   = shift;
+    my $schema = $self->{schema};
+
+    confess kerror() unless verify_args( \%read_infos_args, @_ );
+    my %args     = ( %read_infos_args, @_ );
+    my $folder   = $args{folder};
+    my $filename =
+        defined $args{filename}
+        ? $args{filename}
+        : $folder->latest_gen( 'segments', '.yaml' );
+
     return unless defined $filename;
     my $segs_data = parse_yaml( $folder->slurp_file($filename) );
 
@@ -76,11 +88,20 @@ sub read_infos {
     confess("Unsupported seg infos format: '$segs_data->{format}'")
         unless $segs_data->{format} <= SEG_INFOS_FORMAT;
 
+    # make sure index_interval, skip_interval match Schema
+    for (qw( index_interval skip_interval )) {
+        my $correct = $self->{schema}->$_;
+        next if $segs_data->{$_} == $correct;
+        confess("$_ mismatch: $correct $segs_data->{$_}");
+    }
+
     $self->{seg_counter} = $segs_data->{seg_counter};
     $self->{generation}  = $segs_data->{generation};
+    $self->{ks_creator}  = $segs_data->{ks_creator};
+    $self->{ks_modifier} = $segs_data->{ks_modifier};
 
     # Add any unknown fields to the Schema instance
-    while ( my ( $field, $fspec_class ) = each %{ $segs_data->{fields} }) {
+    while ( my ( $field, $fspec_class ) = each %{ $segs_data->{fields} } ) {
         if ( !$fspec_class->isa("KinoSearch::Schema::FieldSpec") ) {
             confess(  "Attempted to load field '$field' assigned to "
                     . "'$fspec_class', but '$fspec_class' isn't a "
@@ -108,18 +129,24 @@ sub write_infos {
     $self->{generation}++;
     $self->{version}++;
 
+    $self->{ks_creator} ||= $KinoSearch::VERSION;
+    $self->{ks_modifier} = $KinoSearch::VERSION;
+
     # build a hash of field_name => fspec_class pairings
     my %fields
         = map { $_ => ref( $schema->fetch_fspec($_) ) } $schema->all_fields;
 
     # create a YAML-izable data structure
     my %data = (
-        format      => SEG_INFOS_FORMAT,
-        fields      => \%fields,
-        version     => $self->{version},
-        ks_version  => $KinoSearch::VERSION,
-        seg_counter => $self->{seg_counter},
-        generation  => $self->{generation},
+        format         => SEG_INFOS_FORMAT,
+        fields         => \%fields,
+        version        => $self->{version},
+        ks_creator     => $self->{ks_creator},
+        ks_modifier    => $self->{ks_modifier},
+        seg_counter    => $self->{seg_counter},
+        generation     => $self->{generation},
+        index_interval => $schema->index_interval,
+        skip_interval  => $schema->skip_interval,
     );
     my %segments;
     for ( values %{ $self->{infos} } ) {
@@ -164,4 +191,3 @@ See L<KinoSearch> version 0.20.
 
 =end devdocs
 =cut
-
