@@ -5,15 +5,16 @@ package KinoSearch::Index::TermVector;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
+our %instance_vars = (
+    # params / members
+    field         => undef,
+    text          => undef,
+    positions     => [],
+    start_offsets => [],
+    end_offsets   => [],
+);
+
 BEGIN {
-    __PACKAGE__->init_instance_vars(
-        # params / members
-        field         => undef,
-        text          => undef,
-        positions     => [],
-        start_offsets => [],
-        end_offsets   => [],
-    );
     __PACKAGE__->ready_get_set(
         qw(
             field
