@@ -5,7 +5,7 @@ package KinoSearch::Util::ByteBuf;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Obj Exporter );
 
-our @EXPORT_OK = qw( bb_compare bb_less_than ); # testing only
+our @EXPORT_OK = qw( bb_compare bb_less_than );    # testing only
 
 1;
 
@@ -24,11 +24,27 @@ CODE:
     STRLEN len;
     char *ptr = SvPV(sv, len);
     RETVAL = kino_BB_new_str(ptr, len);
-    KINO_UNUSED_VAR(class);
+    CHY_UNUSED_VAR(class);
 }
 OUTPUT: RETVAL
 
-kino_i32_t
+chy_bool_t
+starts_with(self, prefix)
+    kino_ByteBuf *self;
+    kino_ByteBuf *prefix;
+CODE:
+    RETVAL = Kino_BB_Starts_With(self, prefix);
+OUTPUT: RETVAL
+
+chy_bool_t
+ends_with(self, postfix)
+    kino_ByteBuf *self;
+    kino_ByteBuf *postfix;
+CODE:
+    RETVAL = Kino_BB_Ends_With_Str(self, postfix->ptr, postfix->len);
+OUTPUT: RETVAL
+
+chy_i32_t
 bb_compare(bb_a, bb_b)
     kino_ByteBuf *bb_a;
     kino_ByteBuf *bb_b;
@@ -36,7 +52,7 @@ CODE:
     RETVAL = kino_BB_compare(&bb_a, &bb_b);
 OUTPUT: RETVAL
 
-kino_i32_t
+chy_i32_t
 bb_less_than(bb_a, bb_b)
     kino_ByteBuf *bb_a;
     kino_ByteBuf *bb_b;
