@@ -5,10 +5,6 @@ package KinoSearch::Util::DynVirtualTable;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Obj );
 
-BEGIN {
-    __PACKAGE__->init_instance_vars();
-}
-
 1;
 
 __END__
